import {
  createSummarizer,
  CreateSummarizerParams,
  VideoType,
} from '@modal-sh/webvideo-transcript-summary-core';

export interface SummaryResult {
  summary: string;
  normalizedTranscript: string;
  rawTranscript: string;
}

export interface SummaryService {
  summarizeVideoTranscript(params: CreateSummarizerParams): Promise<Partial<SummaryResult>>
}

export class SummaryServiceImpl implements SummaryService {
  constructor(private readonly logger = console) {
    // noop
  }

  summarizeVideoTranscript(params: CreateSummarizerParams) {
    return new Promise<Partial<SummaryResult>>((resolve, reject) => {
      const successEvent = {} as Partial<SummaryResult>;
      let error: Error;
      const summarizer = createSummarizer(params);

      summarizer.on('process', (data) => {
        this.logger.log('process', data);
        if (data.phase === 'success') {
          switch (data.processType) {
            case 'fetch-transcript':
              successEvent.rawTranscript = (
                JSON.parse(data.content as string) as { text: string }[]
              )
                .map((item) => item.text).join(' ');
              break;
            case 'normalize-transcript':
              successEvent.normalizedTranscript = data.content as string;
              break;
            case 'summarize-transcript':
              successEvent.summary = data.content as string;
              break;
            default:
              break;
          }
        }
      });

      summarizer.on('error', (err) => {
        this.logger.log('error', err);
        error = err;
      });

      summarizer.on('end', () => {
        this.logger.log('end');
        if (error) {
          reject(error);
          return;
        }
        resolve(successEvent);
      });

      summarizer.process();
    });
  }
}
